// PictureMgr.cpp : implementation file
//

/*	Portions of Code : were written by  Chensu
	1) Loading from external files
	2) Drawing functions
	Chensu :  www.experts-exchange.com/U.chensu
	I made some changes to some code	Thanks .
*/


/* A known Bug :
It has problems with loading Bitmaps : Error  , But who cares ? ...
It works well with *.jpgs and *wmfs.  Amir.
*/

#include "stdafx.h"
#include "PictureMgr.h"
#include <afxpriv.h>
#include "MemFileEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPictureMgr

CPictureMgr::CPictureMgr()
{
	m_pPicture = NULL;
	pWnd = NULL;
}

CPictureMgr::~CPictureMgr()
{
	ResetContent();
}

/////////////////////////////////////////////////////////////////////////////
// CPictureMgr message handlers

CWnd* CPictureMgr::SetWindow( CWnd *pNewWnd, BOOL bWindowIsView)
{
	CWnd *pTemp = pWnd;
	pWnd = pNewWnd;
	this->bWindowIsView = bWindowIsView;
	return pTemp;
}

void CPictureMgr::ResetContent()
{
	if (m_pPicture) {
		m_pPicture->Release();
		m_pPicture = NULL;
	}
	m_sizeFileSize = 0;
	m_sizeInPix = CSize(0,0);
	m_sizeInHiMetric = CSize(0,0);
	if ((pWnd) && (::IsWindow(pWnd->GetSafeHwnd())))
		pWnd->Invalidate();
}

BOOL CPictureMgr::GetType(short* ptype)
{
	if (!IsExistPicture())  return FALSE;
	HRESULT hr = m_pPicture->get_Type(ptype);
	return SUCCEEDED(hr);
}
BOOL CPictureMgr::GetAttributes(DWORD* pdwAttr)
{
	if (!IsExistPicture())  return FALSE;
	HRESULT hr = m_pPicture->get_Attributes(pdwAttr);
	return SUCCEEDED(hr);
}

BOOL CPictureMgr::LoadPicture( LPCTSTR lpszFileName)
{
	ResetContent();

	// The previous image should be unloaded.
	ASSERT(m_pPicture == NULL);

	CFileStatus fs;
	if (CFile::GetStatus( lpszFileName, fs))
		m_sizeFileSize = fs.m_size; //setting the size of file
	
	USES_CONVERSION;
	HRESULT hr = ::OleLoadPicturePath(const_cast<LPOLESTR>(T2COLE(lpszFileName)),
				NULL, 0, 0, IID_IPicture,
			    reinterpret_cast<LPVOID *>(&m_pPicture));
	
	return SetInfo(hr);
}

/* This function uses directly Resource functions but since resource functions dont really
return HGLOBAL (return HANDLE that is casted to HGLOBAL for compatability reasons
we will have to create our own HGLOBAL and copy data of resources there 
CMemFileEx in the next function GetGlobalFromMemoryFile does this all in 
2 lines moreover : this object is selt contained : it deletes the buffer it creates (using malloc ...)
and theres direct access to the buffer m_lpBuffer (easily copied without Lock ...
*/
HGLOBAL CPictureMgr::GetGlobalFromResourceFunctions( LPCTSTR lpResourceName, LPCTSTR lpResourceType)
{
	HINSTANCE hInst = AfxGetResourceHandle(); 
	HRSRC hRsrc = ::FindResource( hInst, lpResourceName, lpResourceType);
	if (!hRsrc)  return NULL;
	DWORD cbSize = ::SizeofResource(hInst, hRsrc);
	m_sizeFileSize = cbSize;//setting the picture size of file

	HGLOBAL hSrc = ::LoadResource( hInst, hRsrc); 
	if (!hSrc)
		return NULL;

	HGLOBAL hTrueGlobal = ::GlobalAlloc( GHND, cbSize);
	if (!hTrueGlobal) {
		::FreeResource(hSrc);
		return NULL;
	}
	
	LPVOID lpSrcData = ::LockResource(hSrc);
	LPVOID lpTargetData = ::GlobalLock(hTrueGlobal);
	if ((!lpSrcData) || (!lpTargetData)) {
		if (lpSrcData) {
			::UnlockResource(hSrc);
			::FreeResource(hSrc);
		}
		if (lpTargetData) {
			::GlobalUnlock(hTrueGlobal);
			::GlobalFree(hTrueGlobal);
		}
		return NULL;
	}

	//coping the memory of resource to the global handle
	::CopyMemory( lpTargetData, lpSrcData, cbSize);

	//free the resource
	::UnlockResource(hSrc);
	::FreeResource(hSrc);

	::GlobalUnlock(hTrueGlobal);
	return hTrueGlobal;
}

HGLOBAL CPictureMgr::GetGlobalFromMemoryFile(LPCTSTR lpResourceName, LPCTSTR lpResourceType)
{
	CMemFileEx  m_hMemFile;
	if (!m_hMemFile.LoadResource( lpResourceName, lpResourceType))
		return NULL; // resource not loaded

	DWORD cbSize = 0;
	HGLOBAL hGlobal = NULL;
	if (!m_hMemFile.CopyResourceToHGLOBAL( &hGlobal, &cbSize))
		return NULL; //failed to copy into global

	m_sizeFileSize = cbSize;//setting the picture size of file
	return hGlobal;
}


BOOL CPictureMgr::LoadFromResource( LPCTSTR lpResourceName, LPCTSTR lpResourceType, BOOL bUseMemFile)
{
	ResetContent();
	// The previous image should be unloaded.
	ASSERT(m_pPicture == NULL);

	HGLOBAL hGlobal = (bUseMemFile) ? 
			GetGlobalFromMemoryFile( lpResourceName, lpResourceType) : 
			GetGlobalFromResourceFunctions( lpResourceName, lpResourceType);

	return LoadFromGlobal(hGlobal);
}

BOOL CPictureMgr::LoadFromGlobal(HGLOBAL hGlobal)
{
	if (!hGlobal) return FALSE;//hGlobal =0 

	IStream  *pStream = NULL;
	HRESULT hr = ::CreateStreamOnHGlobal( hGlobal, TRUE, &pStream);
	//if SUCCEEDED hGlobal is owned by the IStream object

	if ((!SUCCEEDED(hr)) || (!pStream)) {
		::GlobalFree(hGlobal);	
		return FALSE;
	}

//	 * keep original format
//	 ** size of image (of stream file
//						                                  **	    *
	hr = ::OleLoadPicture( pStream, (LONG)m_sizeFileSize, TRUE,	IID_IPicture, 
			(LPVOID*)&m_pPicture);// m_pPicture);

	if (pStream) //	hGlobal is released
		pStream->Release();

	return SetInfo(hr);
}

void CPictureMgr::Render( CDC *pDC, CSize size, CRect rect)
{
	if (!m_pPicture)  return;
	m_pPicture->Render( pDC->GetSafeHdc(), 0, 0, 
					size.cx, size.cy,
				0, m_sizeInHiMetric.cy,
				m_sizeInHiMetric.cx,-m_sizeInHiMetric.cy, 
				&rect);
}


//called from OnPaint or OnLoad does some optimization 
//really necessary for quick paint
/*
NOTICE: InternalDraw needs a LOT of modifications if you relly want drawing
to be quick , It should support stretching of image (both cases: 
transparent and not, and it also should check  "pDC->GetClipBox" for
optimization ... */
void CPictureMgr::InternalDraw(CDC *pDC)
{
	if (!IsExistPicture())  return;
	if ((!pWnd) || (!::IsWindow(pWnd->GetSafeHwnd())))
		return; //window is not valid

	CRect m_ClientRect;
	::GetClientRect( pWnd->GetSafeHwnd(), &m_ClientRect);

	int width = min( m_sizeInPix.cx, m_ClientRect.Width());
	int height = min( m_sizeInPix.cy, m_ClientRect.Height());

	// get palette
	HPALETTE hPal;
	m_pPicture->get_hPal(reinterpret_cast<OLE_HANDLE *>(&hPal));

	CPalette *pPalOld = NULL;
	if (hPal != NULL) {
		pPalOld = pDC->SelectPalette(CPalette::FromHandle(hPal), FALSE);
		pDC->RealizePalette();
	}

	// transparent?
	DWORD dwAttr;
	if (FAILED(m_pPicture->get_Attributes(&dwAttr)) ||
		(dwAttr & PICTURE_TRANSPARENT))
	{
	// use an off-screen DC to prevent flickering
		CDC MemDC;
		MemDC.CreateCompatibleDC(pDC);
		CBitmap Bmp;
		Bmp.CreateCompatibleBitmap(pDC, m_sizeInPix.cx, m_sizeInPix.cy);
		CBitmap *pBmpOld = MemDC.SelectObject(&Bmp);
		CPalette *pPalMemOld = NULL;
		if (hPal != NULL) {
			pPalMemOld = MemDC.SelectPalette(CPalette::FromHandle(hPal), FALSE);
			MemDC.RealizePalette();
		}

		RECT rc = { 0, 0, m_sizeInPix.cx, m_sizeInPix.cy };
		MemDC.FillSolidRect(&rc, ::GetSysColor(COLOR_WINDOW));

		// display picture using IPicture::Render
		Render( &MemDC, CSize(m_sizeInPix.cx, m_sizeInPix.cy), rc);
		(bWindowIsView) ?
			::BitBlt( pDC->GetSafeHdc(), 0, 0, m_sizeInPix.cx, m_sizeInPix.cy, MemDC.GetSafeHdc(), 0, 0, SRCCOPY)
			:  ::StretchBlt( pDC->GetSafeHdc(), 0, 0, width, height,
					MemDC.GetSafeHdc(),0, 0, m_sizeInPix.cx, m_sizeInPix.cy, SRCCOPY);

		MemDC.SelectObject(pBmpOld);

		if (pPalMemOld != NULL)	{
			ASSERT(hPal != NULL);
			MemDC.SelectPalette(pPalMemOld, FALSE);
		}
	}
	else {
		// display picture using IPicture::Render
		(bWindowIsView) ?
			Render( pDC, CSize(m_sizeInPix.cx, m_sizeInPix.cy), m_ClientRect)
			:	Render( pDC, CSize(width, height), m_ClientRect);
	} //end else

	if (pPalOld != NULL) {
		ASSERT(hPal != NULL);
		pDC->SelectPalette(pPalOld, FALSE);
	}
}

//////////////////////////////////////////////////////////////
///////////
void CPictureMgr::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
	if (!bWindowIsView) //not a view
		return;

	if (pDC->IsPrinting())
	{
		// Set the mapping mode so that the image is printed at the same
		// size as on the screen. Note that images may be too large to be
		// printed completely.
		CDC ScreenDC;
		ScreenDC.CreateIC(_T("DISPLAY"), NULL, NULL, NULL);
		pDC->SetMapMode(MM_ANISOTROPIC);
		pDC->SetWindowExt(ScreenDC.GetDeviceCaps(LOGPIXELSX),
						  ScreenDC.GetDeviceCaps(LOGPIXELSY));
		pDC->SetViewportExt(pDC->GetDeviceCaps(LOGPIXELSX),
							pDC->GetDeviceCaps(LOGPIXELSY));
		pDC->SetWindowOrg(0, 0);
		pDC->SetViewportOrg(0, 0);
	}

} 
// CScrollView ONLY!!!
BOOL CPictureMgr::OnEraseBkgnd(CDC *pDC)
{
	if (!bWindowIsView) return FALSE; //not a view
	CBrush br(::GetSysColor(COLOR_WINDOW));
    ((CScrollView*)pWnd)->FillOutsideRect(pDC, &br);

	return TRUE;
}

// CScrollView ONLY!!!
void CPictureMgr::OnUpdate(CView *pSender, LPARAM lHint, CObject *pHint) 
{
	if (!bWindowIsView) return; //not a view
		
	((CScrollView*)pWnd)->SetScrollSizes(MM_TEXT,
						 m_pPicture != NULL
						 ? CSize( m_sizeInPix.cx, m_sizeInPix.cy)
						 : CSize(0, 0));
}


BOOL CPictureMgr::SetInfo(HRESULT hr)
{
	if (SUCCEEDED(hr) && m_pPicture != NULL) 
	{
		// get width and height of picture
		m_pPicture->get_Width(&m_sizeInHiMetric.cx);
		m_pPicture->get_Height(&m_sizeInHiMetric.cy);

		const int HIMETRIC_PER_INCH = 2540;

		HDC hDCScreen = ::GetDC(NULL);
		ASSERT(hDCScreen != NULL);
		// Pixels per logical inch along width
		const int nPixelsPerInchX = ::GetDeviceCaps(hDCScreen, LOGPIXELSX);
		// Pixels per logical inch along height
		const int nPixelsPerInchY = ::GetDeviceCaps(hDCScreen, LOGPIXELSY);
		::ReleaseDC(NULL, hDCScreen);

		// convert himetric to pixels
		m_sizeInPix.cx = (nPixelsPerInchX * m_sizeInHiMetric.cx +
						  HIMETRIC_PER_INCH / 2) / HIMETRIC_PER_INCH;
		m_sizeInPix.cy = (nPixelsPerInchY * m_sizeInHiMetric.cy +
						  HIMETRIC_PER_INCH / 2) / HIMETRIC_PER_INCH;

		pWnd->Invalidate();
		return TRUE;
	}
	
	// OleLoadPicturePath failed.
	return FALSE;
}

/////////////////////////////////////////////////////////////
/////////  CPictureMgrEx  members

CPictureMgrEx::CPictureMgrEx()
{
}

CPictureMgrEx::~CPictureMgrEx()
{
	ResetContent();
}

void CPictureMgrEx::OnDropFiles(HDROP hDropInfo)
{
	int iCount = ::DragQueryFile( hDropInfo, (UINT)-1, NULL, 0);
    TCHAR  szFile[MAX_PATH*2]; //really really really big space
	for ( int i=0 ; i<iCount ; i++ ) {
        ::DragQueryFile( hDropInfo, i, szFile, sizeof(szFile));
		m_HistoryMgr.Add((LPCTSTR) szFile);
	}
	::DragFinish(hDropInfo);
	hDropInfo = NULL;//to prevent you from trying to free
	//that dead pointer again
}


