// StaticAnim.cpp : implementation file


#include "stdafx.h"
#include "StaticAnim.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////
////// Create and operate StaticAnim in 4 steps :
/*
CStaticAnim anim; //instantiate in class
anim.SubclassDlgItem( IDC_ANIM, this);
anim.SetBitmap( IDB_BITMAP1, 20, 22, TRUE);
anim.Play()
Theres a fifth step : to include the header ...
*/

/*  Static Animation Control
	As you see , this static ctrl uses a dib section as 
a changing image as to mimic behaviour of an animation control.
which is more demanding in matters of resources (animation ctrl 
creates a second thread , animation files (resources) are 
big and increase dramaticly the size of the module)

 There are several drawbacks to this ctrl :
on a heavy job it doesn't recieve WM_TIMER messages and
a)frequency wouldn't be as fast as required 
(on a standard computer (when doing a continouos job) I really didnt notice any performance-fall)
b)you need to get a proper bitmap (on the previous version I used
an ImageList to hold the icons ...(which then were drown)
The application uses some resources I exported from "msoleres.dll"
this dll resides under  \Program Files\Outlook Express\ 
you will find inside many of resources being used by microsoft applications .
(the VB env. also contains a sample that uses a bitmap
to do animation instead of an animation file.

features that are worthwhile to add (will be added next version)
1) SetMenu( CMenu& , BOOL LPFNC(int)) //setting your own menu and a handler to
be called on each menu choose : for you to Invoke an action
2) Transparency (another COLORREF variable to SetBitmap( ... COLORREF regTransparent = RGB(1,1,1))
this unique value is important : in case you want a "common value"
such as black/white/magenta or some ::GetSysColor(...) value
RGB(1,1,1) signals NOT to use transparency.

*/
/////////////////////////////////////////////////////////////////////////////
// CStaticAnim
CStaticAnim::CStaticAnim()
{
	bAdjustWindow2IconSize = TRUE;
	iFrequency = 60 ; // standard frequency
	
	iPosition =0;
	iDirection = UP;
	uTimerID =0 ;
	bPlaying = bEnableContextMenu = FALSE;
	bEnableLink = TRUE;//feature of link and special cursor are disabled
	g_hCursorHand = NULL;
}

CStaticAnim::~CStaticAnim() 
{
}

BOOL CStaticAnim::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style |=	SS_NOTIFY;//in case you forgot to
	return CStatic::PreCreateWindow(cs);
}
void CStaticAnim::PreSubclassWindow() 
{
	ModifyStyle( 0, SS_NOTIFY, 0); //in case you forgot to
	CStatic::PreSubclassWindow();
}
BOOL CStaticAnim::SubclassDlgItem( UINT nID, CWnd* pParent, LPCTSTR lpszLink, BOOL bEnableMenu)
{
	bEnableContextMenu = bEnableMenu;
	BOOL bResult = CStatic::SubclassDlgItem(nID, pParent);
	if (bResult) { //TRUE
		ModifyStyle( 0, SS_NOTIFY, 0);//in case you forgot to
		CreateMenu();
		SetLink(lpszLink);
	}
	return bResult;
}

BEGIN_MESSAGE_MAP(CStaticAnim, CStatic)
	//{{AFX_MSG_MAP(CStaticAnim)
	ON_WM_TIMER()
	ON_WM_PAINT()
	ON_WM_DESTROY()
	ON_WM_CONTEXTMENU()
	ON_WM_SETCURSOR()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(WM_USER+1,WM_USER+2,OnMenuCommand)
END_MESSAGE_MAP()

BOOL CStaticAnim::CreateMenu()
{
	if (m_hMenu.GetSafeHmenu())
		m_hMenu.DestroyMenu();
	if (m_hMenu.CreatePopupMenu()) {
		m_hMenu.AppendMenu(MF_STRING,1,_T("Play"));
		m_hMenu.AppendMenu(MF_STRING,2,_T("Pause"));
		m_hMenu.AppendMenu(MF_SEPARATOR,0,_T(""));
		int checked = (bPlaying) ? 1 : 2 ;
		m_hMenu.CheckMenuRadioItem(1,2,checked ,MF_BYCOMMAND);
	}
	return (BOOL) m_hMenu.GetSafeHmenu();
}

void CStaticAnim::AdjustSize( int cx, int cy, BOOL bAdjustWindowSize)
{
	m_ImageSize = CSize( cx, cy);
	ASSERT(cx>0);
	ASSERT(cy>0); //measurements must be valid

	bAdjustWindow2IconSize = bAdjustWindowSize ;
	if (bAdjustWindow2IconSize) {
		CRect rect;
		GetWindowRect(&rect);

		CPoint point(rect.TopLeft()); //ressetting the control
		//according the top-left spot (we may resset according
		//another spot too : top-right is useful(for right alignment))
		::ScreenToClient( ::GetParent(m_hWnd), &point);
		MoveWindow( point.x, point.y, cx, cy);
	}
}

BOOL CStaticAnim::SetBitmap( LPCTSTR lpszResourceName, int cx, int cy, BOOL bAdjustWindowSize)
{
	AdjustSize( cx, cy, bAdjustWindowSize);
	HBITMAP hbm = (HBITMAP)::LoadImage( 
		AfxGetInstanceHandle(),
		lpszResourceName,	IMAGE_BITMAP,
		0, 0, LR_CREATEDIBSECTION);

	if (!hbm)	return FALSE;
	if (!m_bitmap.Attach(hbm)) {
		::DeleteObject(hbm);
		return FALSE;
	}

	BITMAP bm;
	m_bitmap.GetObject( sizeof(BITMAP), &bm);
	m_BitmapSize = CSize( bm.bmWidth, bm.bmHeight);
	
	CClientDC dc(NULL);
	if (dc.GetDeviceCaps(RASTERCAPS) & RC_PALETTE == 0)
		return TRUE;

	if (m_bitmap.GetSafeHandle()) {
		int nColors;
		DIBSECTION  ds;
		m_bitmap.GetObject(sizeof(DIBSECTION),&ds);

		if (ds.dsBmih.biClrUsed != 0)
			nColors = ds.dsBmih.biClrUsed;
		else nColors = 1 << ds.dsBmih.biBitCount;

		if (nColors > 256)
			m_palette.CreateHalftonePalette(&dc);
		else { //nColors < 256)
			RGBQUAD  *pRGB = new RGBQUAD[nColors];
			if (!pRGB)  return FALSE;

			CDC memDC;
			memDC.CreateCompatibleDC(&dc);
			CBitmap *pOldBitmap = memDC.SelectObject(&m_bitmap);
			::GetDIBColorTable( memDC.GetSafeHdc(), 0, nColors, pRGB);
			memDC.SelectObject(pOldBitmap);

			UINT nSize = sizeof(LOGPALETTE) + 
				(sizeof(PALETTEENTRY) * (nColors+1));
			LOGPALETTE *pLP = (LOGPALETTE*) new BYTE[nSize];
			pLP->palVersion = 0x300;
			pLP->palNumEntries = nColors;

			for (int i=0 ; i<nColors ; i++) {
				pLP->palPalEntry[i].peBlue = pRGB[i].rgbRed;
				pLP->palPalEntry[i].peGreen = pRGB[i].rgbGreen;
				pLP->palPalEntry[i].peBlue = pRGB[i].rgbBlue;
				pLP->palPalEntry[i].peFlags = 0;
			}
			m_palette.CreatePalette(pLP);
			delete [] pLP;
			delete [] pRGB;
		} // (nColors < 256)
	} // got bitmap

	//Find the count of images
	iImageCount = (m_ImageSize.cx == m_BitmapSize.cx) ? (m_BitmapSize.cy/m_ImageSize.cy)
			: (m_BitmapSize.cx/m_ImageSize.cx);

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CStaticAnim message handlers
BOOL CStaticAnim::Play()
{
	if (uTimerID)  return TRUE; //already got timer
	for ( UINT uTimerEvent = 1000 ; ((uTimerEvent<3000) && (!uTimerID)) ; uTimerEvent++)
		if (SetTimer( uTimerEvent, (UINT)iFrequency, NULL)) {
			uTimerID = uTimerEvent;
			bPlaying = TRUE;
			return TRUE;
		}
	return FALSE;
}
BOOL CStaticAnim::Pause() 
{
	if ((uTimerID) && (KillTimer(uTimerID)))
		uTimerID = 0;
	bPlaying = FALSE;
	return TRUE;
}
int CStaticAnim::SetFrequency ( int iNewFrequency) 
{
	if ((iNewFrequency>=0) && (iNewFrequency<=40000))
		iFrequency = iNewFrequency;
	
	ASSERT(iFrequency>0);//must be valid
	if (bPlaying) { //reset of play (with new frequency)
		Pause();
		Play();
	}
	return iFrequency;
}

void CStaticAnim::OnTimer(UINT nIDEvent) 
{
	if (uTimerID == nIDEvent)
		Step(TRUE);
	else CStatic::OnTimer(nIDEvent);
}

//checks the direction that is entered
DIRECTION CStaticAnim::SetDirection( DIRECTION iNewDirection)
{
	if ((iNewDirection == UP) || (iNewDirection == DOWN))
		iDirection = iNewDirection;
	return GetDirection();
}
int CStaticAnim::SetPosition( int iNewPosition) 
{
	if ((iNewPosition >= 0) && (iNewPosition < iImageCount))
		iPosition = iNewPosition;
	return GetPosition();
}

// This function moves to the next place and draws the next image
// (plays 1 step in animation)
BOOL CStaticAnim::Step(BOOL bMoveNext)
{
	int iCurrentPosition = GetPosition();
	int iCurrentDirection = GetDirection();

	iCurrentPosition++;
	if (iCurrentPosition>= iImageCount)
		iCurrentPosition = 0;
	SetPosition(iCurrentPosition);

	Invalidate();
	UpdateWindow();
	return TRUE;
}
void CStaticAnim::OnMenuCommand(UINT nID) //to enable menus
{
}


BOOL CStaticAnim::SetMenu( CMenu &menu, BOOL bDetachOriginal)
{
	HMENU hMenu = menu.GetSafeHmenu();
	if (!hMenu)  return FALSE; //no menu previous menu stays
	
	if (m_hMenu.GetSafeHmenu())
		m_hMenu.DestroyMenu();

	if (m_hMenu.Attach(hMenu)) {
		if (bDetachOriginal)
			menu.Detach();
		return TRUE;
	}
	return FALSE;
}


void CStaticAnim::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	if (!m_bitmap.GetSafeHandle())
		return; //no bitmap (nothing to draw)
	
	int iCurrentImage = GetPosition();

	//checking whether picture is horizontal or vertical
	//in order to find start position of first bitmap
	CSize m_StartPoint;
	BOOL bHorz = (m_BitmapSize.cx > m_BitmapSize.cy);
	m_StartPoint = (bHorz) ?
		CSize( iCurrentImage * m_ImageSize.cx, 0) 
		: CSize( 0, iCurrentImage * m_ImageSize.cy); 

	CPalette *pOldPalette = dc.SelectPalette(&m_palette,FALSE);
	dc.RealizePalette();

	CDC memDC;
	memDC.CreateCompatibleDC(&dc);
	CBitmap *pOldBitmap = memDC.SelectObject(&m_bitmap);

	::BitBlt( dc.GetSafeHdc(),0, 0, m_ImageSize.cx, m_ImageSize.cy, 
		memDC.GetSafeHdc(), m_StartPoint.cx, m_StartPoint.cy, SRCCOPY);

	memDC.SelectObject(pOldBitmap);
	dc.SelectPalette( pOldPalette, FALSE);
}
void CStaticAnim::OnDestroy() 
{
	Pause(); // to kill timer
	::DestroyCursor(g_hCursorHand);
	CStatic::OnDestroy();
}

//default implementation of menu
void CStaticAnim::DoContextMenu(CWnd* pWnd, CPoint point)
{
	if (!m_hMenu.GetSafeHmenu()) return;//no menu

	/// Default implementation of menus
	int checked = (bPlaying) ? 1 : 2 ;
	m_hMenu.CheckMenuRadioItem(1,2,checked ,MF_BYCOMMAND);

	int iCmd = (int) m_hMenu.TrackPopupMenu( 
			TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RETURNCMD ,
				point.x , point.y ,this );
	
	switch (iCmd)
	{
	case 1:	Play();		
		break;
	case 2:	Pause();
		break;
	}
	checked = (bPlaying) ? 1 : 2 ;
	m_hMenu.CheckMenuRadioItem(1,2,checked ,MF_BYCOMMAND);
}
void CStaticAnim::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (!bEnableContextMenu) return; // Context menu is disabled
	DoContextMenu( pWnd, point);
}


////////// SetLink and behaviour ...
void CStaticAnim::SetLink( LPCTSTR lpszLink)
{
	bEnableLink = (lpszLink) ? TRUE : FALSE;
	if (bEnableLink)
		strLink = lpszLink;
}

BOOL CStaticAnim::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if (bEnableLink) {
		if (g_hCursorHand == NULL) {
			static bTriedOnce = FALSE;
			if (!bTriedOnce) {
				EnableToolTips(TRUE);
				CString windir;
				::GetWindowsDirectory(windir.GetBuffer(MAX_PATH), MAX_PATH);
				windir.ReleaseBuffer();
				windir += _T("\\winhlp32.exe");
				HMODULE hModule = LoadLibrary(windir);
				if (hModule) {
					g_hCursorHand =
						CopyCursor(::LoadCursor(hModule, MAKEINTRESOURCE(106)));
				}
				::FreeLibrary(hModule);
				bTriedOnce = TRUE;
			}
		}
		if (g_hCursorHand) {
			::SetCursor(g_hCursorHand);
			return TRUE;
		}
	} // bEnableLink = TRUE

	//here cursor should be arrow of hand-cursor not created
	return CStatic::OnSetCursor(pWnd, nHitTest, message);
}

//if Navigation features are available then - this causes navigation to
//the link in strLink
void CStaticAnim::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (bEnableLink) {
		HINSTANCE h = (strLink.IsEmpty()) ? NULL :
			ShellExecute(0, _T("open"), strLink, 0, 0, SW_SHOWNORMAL);
		//you may check returned value
		// (h>(HINSTANCE)32) ? TRUE : FALSE ;
		return;
	}

	//Navigation not available 
	CStatic::OnLButtonDown(nFlags, point);
}
